'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var PropTypes = require('prop-types');
var ReactDOM = require('react-dom');
var cssElementQueries = require('css-element-queries');

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css = ".spaces-centered-vertically {\r\n\tposition: relative;\r\n\ttop: 50%;\r\n\ttransform: translateY(-50%);\r\n}\r\n\r\n.spaces-centered {\r\n\tposition: relative;\r\n\ttop: 50%;\r\n\ttransform: translateY(-50%);\r\n\ttext-align: center;\r\n}\r\n\r\n.spaces-clearfix:after {\r\n\tcontent: \"\";\r\n\tdisplay: table;\r\n\tclear: both;\r\n}\r\n\r\n.spaces-fixedsize-layout {\r\n\toverflow: hidden;\r\n\tposition: relative;\r\n}\r\n\r\n.spaces-fullpage-layout {\r\n\tposition: fixed;\r\n\toverflow: hidden;\r\n}\r\n\r\n.spaces-resize-handle {\r\n\tposition: absolute;\r\n\tz-index: 9999;\r\n\ttouch-action: none;\r\n}\r\n\r\n.spaces-resize-handle.resize-left {\r\n\ttop: 0;\r\n\tbottom: 0;\r\n\tleft: 0;\r\n\tcursor: w-resize;\r\n}\r\n\r\n.spaces-resize-handle.resize-right {\r\n\ttop: 0;\r\n\tbottom: 0;\r\n\tright: 0;\r\n\tcursor: e-resize;\r\n}\r\n\r\n.spaces-resize-handle.resize-top {\r\n\ttop: 0;\r\n\tleft: 0;\r\n\tright: 0;\r\n\tcursor: n-resize;\r\n}\r\n\r\n.spaces-resize-handle.resize-bottom {\r\n\tbottom: 0;\r\n\tleft: 0;\r\n\tright: 0;\r\n\tcursor: s-resize;\r\n}\r\n\r\n.spaces-space {\r\n\tposition: absolute;\r\n\toverflow: hidden;\r\n\ttouch-action: none;\r\n\tbox-sizing: border-box;\r\n}\r\n\r\n.spaces-space.resizing .spaces-space {\r\n\ttransition: inherit !important;\r\n}\r\n\r\n.spaces-space .spaces-space-inner {\r\n\tposition: absolute;\r\n\tleft: 0;\r\n\ttop: 0;\r\n\tright: 0;\r\n\tbottom: 0;\r\n\tbox-sizing: border-box;\r\n}\r\n\r\n.spaces-space.scrollable .spaces-space-inner, \r\n.spaces-space.scrollable-a {\r\n\toverflow: auto;\r\n\ttouch-action: auto;\r\n}";
styleInject(css);

var Centered = function (props) { return (React.createElement("div", { className: "spaces-centered" }, props.children)); };
var CenteredVertically = function (props) { return (React.createElement("div", { className: "spaces-centered-vertically" }, props.children)); };

var ClearFix = function (props) { return (React.createElement("div", { className: "spaces-clearfix" }, props.children)); };

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var SpaceContext = React.createContext(null);
var SpaceLayerContext = React.createContext(null);
var SpaceInfoContext = React.createContext(null);

var ResizeType;
(function (ResizeType) {
    ResizeType["Left"] = "resize-left";
    ResizeType["Right"] = "resize-right";
    ResizeType["Top"] = "resize-top";
    ResizeType["Bottom"] = "resize-bottom";
})(ResizeType || (ResizeType = {}));
(function (AnchorType) {
    AnchorType["Left"] = "anchor-left";
    AnchorType["Right"] = "anchor-right";
    AnchorType["Top"] = "anchor-top";
    AnchorType["Bottom"] = "anchor-bottom";
})(exports.AnchorType || (exports.AnchorType = {}));
var AnchorTypes = [
    exports.AnchorType.Left,
    exports.AnchorType.Right,
    exports.AnchorType.Bottom,
    exports.AnchorType.Top
];
var AnchorToResizeTypeMap = {
    "anchor-left": ResizeType.Right,
    "anchor-right": ResizeType.Left,
    "anchor-bottom": ResizeType.Top,
    "anchor-top": ResizeType.Bottom
};
(function (CenterType) {
    CenterType["None"] = "none";
    CenterType["Vertical"] = "vertical";
    CenterType["HorizontalVertical"] = "horizontalVertical";
})(exports.CenterType || (exports.CenterType = {}));
var publicProps = {
    id: PropTypes.string,
    className: PropTypes.string,
    style: PropTypes.oneOfType([PropTypes.object, PropTypes.array]),
    scrollable: PropTypes.bool,
    trackSize: PropTypes.bool,
    centerContent: PropTypes.oneOf([exports.CenterType.None, exports.CenterType.Vertical, exports.CenterType.HorizontalVertical]),
    as: PropTypes.string,
    zIndex: PropTypes.number,
    onClick: PropTypes.func,
    onMouseDown: PropTypes.func,
    onMouseEnter: PropTypes.func,
    onMouseLeave: PropTypes.func,
    onMouseMove: PropTypes.func,
    onTouchStart: PropTypes.func,
    onTouchMove: PropTypes.func,
    onTouchEnd: PropTypes.func
};
var privateProps = {
    anchorSize: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    anchor: PropTypes.oneOf([exports.AnchorType.Bottom, exports.AnchorType.Left, exports.AnchorType.Right, exports.AnchorType.Top]),
    resizable: PropTypes.bool,
    order: PropTypes.number,
    topMost: PropTypes.bool
};
var anchoredProps = {
    size: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired,
    order: PropTypes.number
};
var resizableProps = {
    handleSize: PropTypes.number,
    overlayHandle: PropTypes.bool,
    minimumSize: PropTypes.number,
    maximumSize: PropTypes.number,
    onResizeStart: PropTypes.func,
    onResizeEnd: PropTypes.func
};
var positionedProps = {
    left: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    top: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    right: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    bottom: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    width: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    height: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
    resizable: PropTypes.bool
};
var allProps = __assign({}, publicProps, privateProps, resizableProps, positionedProps);

var getSizeString = function (size) { return typeof (size) === "string" ? size : "" + size + (size !== 0 ? "px" : ""); };
var isHorizontalSpace = function (anchorType) {
    return anchorType && (anchorType === exports.AnchorType.Left || anchorType === exports.AnchorType.Right);
};
var isVerticalSpace = function (anchorType) {
    return anchorType && (anchorType === exports.AnchorType.Top || anchorType === exports.AnchorType.Bottom);
};
function shortuuid() {
    var firstPart = (Math.random() * 46656) | 0;
    var secondPart = (Math.random() * 46656) | 0;
    return ("000" + firstPart.toString(36)).slice(-3) + ("000" + secondPart.toString(36)).slice(-3);
}
var initialState = function (props) { return ({
    id: props.id || "s" + shortuuid(),
    children: []
}); };
var cssValue = function (value, adjusted) {
    return adjusted ?
        "calc(" + getSizeString(value || 0) + " + " + getSizeString(adjusted) + ")" :
        getSizeString(value || 0);
};

var recalcSpaces = function (spaces) {
    var _loop_1 = function (sidx) {
        var space = spaces[sidx];
        var adjustedTop = [];
        var adjustedLeft = [];
        var adjustedRight = [];
        var adjustedBottom = [];
        if (space.anchorType === undefined) {
            space.left = 0;
            space.top = 0;
            space.right = 0;
            space.bottom = 0;
        }
        else {
            space.left = space.anchorType !== exports.AnchorType.Right ? 0 : undefined;
            space.top = space.anchorType !== exports.AnchorType.Bottom ? 0 : undefined;
            space.right = space.anchorType !== exports.AnchorType.Left ? 0 : undefined;
            space.bottom = space.anchorType !== exports.AnchorType.Top ? 0 : undefined;
        }
        var _loop_2 = function (at) {
            var spaceTakers = spaces
                .filter(function (t) { return t.anchorType !== undefined && t.zIndex === space.zIndex && t.anchorType === AnchorTypes[at]; })
                .sort(function (a, b) { return a.order - b.order; });
            for (var i = 0; i < spaceTakers.length; i++) {
                var t = spaceTakers[i];
                if (t.id !== space.id) {
                    var adjustedSize = t.adjustedSize !== 0 ? " + " + getSizeString(t.adjustedSize) : "";
                    if (space.anchorType === undefined) {
                        if (t.anchorType === exports.AnchorType.Top && t.anchorSize) {
                            adjustedTop.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                        else if (t.anchorType === exports.AnchorType.Left && t.anchorSize) {
                            adjustedLeft.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                        else if (t.anchorType === exports.AnchorType.Bottom && t.anchorSize) {
                            adjustedBottom.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                        else if (t.anchorType === exports.AnchorType.Right && t.anchorSize) {
                            adjustedRight.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                    }
                    else {
                        if (t.anchorType === exports.AnchorType.Top && t.anchorSize && space.top !== undefined) {
                            adjustedTop.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                        else if (t.anchorType === exports.AnchorType.Left && t.anchorSize && space.left !== undefined) {
                            adjustedLeft.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                        else if (t.anchorType === exports.AnchorType.Bottom && t.anchorSize && space.bottom !== undefined) {
                            adjustedBottom.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                        else if (t.anchorType === exports.AnchorType.Right && t.anchorSize && space.right !== undefined) {
                            adjustedRight.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                    }
                }
                else {
                    break;
                }
            }
        };
        for (var at = 0; at < AnchorTypes.length; at++) {
            _loop_2(at);
        }
        [
            { adjusted: adjustedTop, setter: function (value) { return space.top = value; } },
            { adjusted: adjustedBottom, setter: function (value) { return space.bottom = value; } },
            { adjusted: adjustedLeft, setter: function (value) { return space.left = value; } },
            { adjusted: adjustedRight, setter: function (value) { return space.right = value; } }
        ].map(function (x) { return x.adjusted.length > 0 && x.setter("calc(" + x.adjusted.join(" + ") + ")"); });
    };
    for (var sidx = 0; sidx < spaces.length; sidx++) {
        _loop_1(sidx);
    }
    return spaces;
};
var registerSpace = function (context, space) {
    var existing = context.children.find(function (t) { return t.id === space.id; });
    if (!existing) {
        context.updateChildren(recalcSpaces(context.children.concat([space])));
        return space;
    }
    else {
        existing.order = space.order;
        existing.anchorType = space.anchorType;
        existing.anchorSize = space.anchorSize;
        return existing;
    }
};
var removeSpace = function (context, id) {
    context.updateChildren(recalcSpaces(context.children.filter(function (t) { return t.id !== id; })));
};
var updateSpace = function (context, id, delta) {
    context.updateChildren(recalcSpaces(context.children.map(function (t) { return t.id === id ? __assign({}, t, delta) : t; })));
};
var createSpaceContext = function (children, updateChildren, currentSpace, parent) {
    var context = {
        level: parent ? parent.level + 1 : 0,
        children: children,
        updateChildren: updateChildren
    };
    return context;
};

var HeadStyles = function (props) {
    var spaces = props.spaces;
    if (spaces.length > 0) {
        var styles = [];
        for (var i = 0; i < spaces.length; i++) {
            var space = spaces[i];
            var css = [];
            var style = {
                left: (space.left !== undefined ? cssValue(space.left, space.adjustedLeft) : undefined),
                top: (space.top !== undefined ? cssValue(space.top, space.adjustedTop) : undefined),
                right: (space.right !== undefined ? cssValue(space.right, space.adjustedLeft) : undefined),
                bottom: (space.bottom !== undefined ? cssValue(space.bottom, space.adjustedTop) : undefined),
                width: isHorizontalSpace(space.anchorType) ? cssValue(space.anchorSize, space.adjustedSize) : space.width,
                height: isVerticalSpace(space.anchorType) ? cssValue(space.anchorSize, space.adjustedSize) : space.height,
                zIndex: space.zIndex
            };
            if (style.left) {
                css.push("left: " + style.left + ";");
            }
            if (style.top) {
                css.push("top: " + style.top + ";");
            }
            if (style.right) {
                css.push("right: " + style.right + ";");
            }
            if (style.bottom) {
                css.push("bottom: " + style.bottom + ";");
            }
            if (style.width) {
                css.push("width: " + style.width + ";");
            }
            if (style.height) {
                css.push("height: " + style.height);
            }
            if (style.zIndex) {
                css.push("z-index: " + style.zIndex);
            }
            if (css.length > 0) {
                styles.push("#" + space.id + " { " + css.join(' ') + " }");
            }
        }
        if (styles.length === 0) {
            return null;
        }
        return ReactDOM.createPortal(React.createElement("style", null, styles.join(' ')), window.document.head);
    }
    return null;
};

var Fixed = function (props) {
    var _a = React.useState([]), children = _a[0], setChildren = _a[1];
    var style = __assign({}, props.style, {
        width: props.width,
        height: props.height
    });
    return (React.createElement("div", { className: "spaces-fixedsize-layout" + (props.className ? " " + props.className : ""), style: style },
        React.createElement(HeadStyles, { spaces: children }),
        React.createElement(SpaceContext.Provider, { value: createSpaceContext(children, setChildren) }, props.children)));
};
Fixed.propTypes = {
    className: PropTypes.string,
    style: PropTypes.oneOfType([PropTypes.object, PropTypes.array]),
    width: PropTypes.number,
    height: PropTypes.number.isRequired
};

var ViewPort = function (props) {
    var _a = React.useState([]), children = _a[0], setChildren = _a[1];
    return (React.createElement("div", { className: "spaces-fullpage-layout" + (props.className ? " " + props.className : ""), style: {
            left: props.left || 0,
            top: props.top || 0,
            right: props.right || 0,
            bottom: props.bottom || 0
        } },
        React.createElement(HeadStyles, { spaces: children }),
        React.createElement(SpaceContext.Provider, { value: createSpaceContext(children, setChildren) }, props.children)));
};
ViewPort.propTypes = {
    className: PropTypes.string,
    left: PropTypes.number,
    top: PropTypes.number,
    right: PropTypes.number,
    bottom: PropTypes.number
};

var Layer = function (props) {
    return React.createElement(SpaceLayerContext.Provider, { value: props.zIndex }, props.children);
};

var Info = function (props) { return (React.createElement(SpaceInfoContext.Consumer, null, function (info) { return props.children(info); })); };

var useSpace = function (props, divElementRef) {
    var parentContext = React.useContext(SpaceContext);
    if (!parentContext) {
        throw new Error("No top-level space: There must be a <ViewPort /> or <Fixed /> ancestor space");
    }
    var _a = React.useState(initialState(props)), state = _a[0], changeState = _a[1];
    var _b = React.useState(null), currentSize = _b[0], setCurrentSize = _b[1];
    var resizeSensor = React.useRef();
    var onRemove = React.useRef();
    var setState = function (stateDelta) { return changeState(function (prev) { return (__assign({}, prev, stateDelta)); }); };
    var layerContext = React.useContext(SpaceLayerContext);
    var currentZIndex = props.zIndex || layerContext || 0;
    var updateCurrentSize = React.useCallback(function () {
        if (divElementRef.current) {
            var rect = divElementRef.current.getBoundingClientRect();
            setCurrentSize({
                width: rect.width,
                height: rect.height
            });
        }
    }, []);
    var space = registerSpace(parentContext, {
        id: state.id,
        zIndex: currentZIndex,
        order: props.order === undefined ? 1 : props.order,
        anchorType: props.anchor,
        anchorSize: props.anchorSize || 0,
        adjustedSize: 0,
        adjustedLeft: 0,
        adjustedTop: 0,
        width: props.anchor && isHorizontalSpace(props.anchor) ? props.anchorSize || 0 : props.width,
        height: props.anchor && isVerticalSpace(props.anchor) ? props.anchorSize || 0 : props.height
    });
    React.useEffect(function () {
        setCurrentSize(null);
    }, [parentContext.children.length]);
    // Deal with property changes to size / zIndex / anchoring 
    React.useEffect(function () {
        updateSpace(parentContext, state.id, {
            width: isHorizontalSpace(props.anchor) ? props.anchorSize || 0 : props.width,
            height: isVerticalSpace(props.anchor) ? props.anchorSize || 0 : props.height,
            adjustedSize: 0
        });
    }, [props.anchorSize]);
    React.useEffect(function () {
        updateSpace(parentContext, state.id, {
            left: props.anchor !== exports.AnchorType.Right ? props.left || 0 : undefined,
            top: props.anchor !== exports.AnchorType.Bottom ? props.top || 0 : undefined,
            right: props.anchor !== exports.AnchorType.Left ? props.right || 0 : undefined,
            bottom: props.anchor !== exports.AnchorType.Top ? props.bottom || 0 : undefined,
            width: isHorizontalSpace(props.anchor) ? props.anchorSize || 0 : props.width,
            height: isVerticalSpace(props.anchor) ? props.anchorSize || 0 : props.height
        });
    }, [props.left, props.top, props.bottom, props.right, props.width, props.height, props.anchor]);
    React.useEffect(function () {
        updateSpace(parentContext, state.id, {
            zIndex: currentZIndex
        });
    }, [currentZIndex]);
    // Setup / cleanup
    React.useEffect(function () {
        if (divElementRef.current) {
            if (props.trackSize) {
                resizeSensor.current = new cssElementQueries.ResizeSensor(divElementRef.current, function (size) { return setCurrentSize({
                    width: size.width,
                    height: size.height
                }); });
            }
        }
        updateCurrentSize();
        var cleanup = function () {
            resizeSensor.current && resizeSensor.current.detach();
            onRemove.current && onRemove.current();
        };
        return cleanup;
    }, []);
    React.useEffect(function () {
        if (!currentSize) {
            updateCurrentSize();
        }
    });
    onRemove.current = React.useCallback(function () {
        removeSpace(parentContext, state.id);
    }, []);
    var currentContext = createSpaceContext(state.children, function (children) { return setState({ children: children }); }, space, parentContext);
    return {
        space: space,
        parentContext: parentContext,
        currentContext: currentContext,
        currentSize: currentSize || { width: 0, height: 0 }
    };
};

var ResizeHandle = function (props) {
    return (React.createElement("div", { style: { width: props.width, height: props.height }, className: "spaces-resize-handle " + props.type, onMouseDown: props.onMouseDown, onTouchStart: props.onTouchStart }));
};
ResizeHandle.propTypes = {
    type: PropTypes.oneOf([ResizeType.Bottom, ResizeType.Left, ResizeType.Right, ResizeType.Top]).isRequired,
    width: PropTypes.number,
    height: PropTypes.number
};

function throttle(callback, limit) {
    var wait = false; // Initially, we're not waiting
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (!wait) { // If we're not waiting
            callback.apply(void 0, args); // Execute users function
            wait = true; // Prevent future invocations
            setTimeout(function () {
                wait = false; // And allow future invocations
            }, limit);
        }
    };
}

var RESIZE_THROTTLE = 5;
var Fill = function (props) { return React.createElement(SpaceInternal, __assign({}, props)); };
Fill.propTypes = publicProps;
var Top = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Top, anchorSize: props.size })); };
Top.propTypes = __assign({}, publicProps, anchoredProps);
var Left = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Left, anchorSize: props.size })); };
Left.propTypes = __assign({}, publicProps, anchoredProps);
var Bottom = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Bottom, anchorSize: props.size })); };
Bottom.propTypes = __assign({}, publicProps, anchoredProps);
var Right = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Right, anchorSize: props.size })); };
Right.propTypes = __assign({}, publicProps, anchoredProps);
var TopResizable = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Top, anchorSize: props.size, resizable: true })); };
TopResizable.propTypes = __assign({}, publicProps, anchoredProps, resizableProps);
var LeftResizable = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Left, anchorSize: props.size, resizable: true })); };
LeftResizable.propTypes = __assign({}, publicProps, anchoredProps, resizableProps);
var BottomResizable = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Bottom, anchorSize: props.size, resizable: true })); };
BottomResizable.propTypes = __assign({}, publicProps, anchoredProps, resizableProps);
var RightResizable = function (props) { return React.createElement(SpaceInternal, __assign({}, props, { anchor: exports.AnchorType.Right, anchorSize: props.size, resizable: true })); };
RightResizable.propTypes = __assign({}, publicProps, anchoredProps, resizableProps);
var Positioned = function (props) { return React.createElement(SpaceInternal, __assign({}, props)); };
RightResizable.propTypes = __assign({}, publicProps, resizableProps, positionedProps);
var SpaceInternal = React.memo(function (props) {
    var divElementRef = React.useRef();
    var _a = useSpace(props, divElementRef), space = _a.space, parentContext = _a.parentContext, currentContext = _a.currentContext, currentSize = _a.currentSize;
    var handleSize = props.handleSize === undefined ? 5 : props.handleSize;
    var overlayHandle = props.overlayHandle !== undefined ? props.overlayHandle : true;
    var resizeType = undefined;
    var resizeHandle = undefined;
    var onResizeEnd = React.useCallback(function () {
        if (divElementRef.current) {
            var currentRect = divElementRef.current.getBoundingClientRect();
            props.onResizeEnd && props.onResizeEnd(Math.floor(resizeType === ResizeType.Left || resizeType === ResizeType.Right ? currentRect.width : currentRect.height));
        }
    }, []);
    if (parentContext && props.anchor && props.resizable) {
        resizeType = AnchorToResizeTypeMap[props.anchor];
        var resizeHandleWidth = resizeType === ResizeType.Left || resizeType === ResizeType.Right ? handleSize : undefined;
        var resizeHandleHeight = resizeType === ResizeType.Top || resizeType === ResizeType.Bottom ? handleSize : undefined;
        var onResize_1 = function (originalX, originalY, x, y, minimumAdjust, maximumAdjust) {
            var adjustmentX = Math.min(Math.max(resizeType === ResizeType.Left ? originalX - x : x - originalX, minimumAdjust), maximumAdjust === undefined ? 999999 : maximumAdjust);
            var adjustmentY = Math.min(Math.max(resizeType === ResizeType.Top ? originalY - y : y - originalY, minimumAdjust), maximumAdjust === undefined ? 999999 : maximumAdjust);
            var adjustment = isHorizontalSpace(props.anchor) ? adjustmentX : adjustmentY;
            if (adjustment !== space.adjustedSize) {
                updateSpace(parentContext, space.id, { adjustedSize: adjustment });
            }
        };
        var startTouchResize_1 = function (e) {
            if (!divElementRef.current) {
                return;
            }
            var rect = divElementRef.current.getBoundingClientRect();
            var size = isHorizontalSpace(props.anchor) ? rect.width : rect.height;
            var originalTouchX = resizeType === ResizeType.Left ? e.touches[0].pageX + space.adjustedSize : e.touches[0].pageX - space.adjustedSize;
            var originalTouchY = resizeType === ResizeType.Top ? e.touches[0].pageY + space.adjustedSize : e.touches[0].pageY - space.adjustedSize;
            var minimumAdjust = (props.minimumSize === undefined ? 20 : props.minimumSize) - size + space.adjustedSize;
            var maximumAdjust = props.maximumSize ? (props.maximumSize - size + space.adjustedSize) : undefined;
            var lastX = 0;
            var lastY = 0;
            var moved = false;
            var touchResize = function (x, y) { return onResize_1(originalTouchX, originalTouchY, x, y, minimumAdjust, maximumAdjust); };
            var throttledTouchResize = throttle(touchResize, RESIZE_THROTTLE);
            var withPreventDefault = function (e) {
                moved = true;
                lastX = e.touches[0].pageX;
                lastY = e.touches[0].pageY;
                e.preventDefault();
                e.stopImmediatePropagation();
                throttledTouchResize(lastX, lastY);
            };
            var removeListener = function () {
                if (moved) {
                    touchResize(lastX, lastY);
                }
                window.removeEventListener('touchmove', withPreventDefault);
                window.removeEventListener('touchend', removeListener);
                onResizeEnd();
            };
            window.addEventListener('touchmove', withPreventDefault);
            window.addEventListener('touchend', removeListener);
            e.preventDefault();
            e.stopPropagation();
            props.onResizeStart && props.onResizeStart();
        };
        var startResize_1 = function (e) {
            if (!divElementRef.current) {
                return;
            }
            var rect = divElementRef.current.getBoundingClientRect();
            var size = isHorizontalSpace(props.anchor) ? rect.width : rect.height;
            var originalMouseX = resizeType === ResizeType.Left ? e.pageX + space.adjustedSize : e.pageX - space.adjustedSize;
            var originalMouseY = resizeType === ResizeType.Top ? e.pageY + space.adjustedSize : e.pageY - space.adjustedSize;
            var minimumAdjust = (props.minimumSize === undefined ? 20 : props.minimumSize) - size + space.adjustedSize;
            var maximumAdjust = props.maximumSize ? (props.maximumSize - size + space.adjustedSize) : undefined;
            var lastX = 0;
            var lastY = 0;
            var moved = false;
            var mouseResize = function (x, y) { return onResize_1(originalMouseX, originalMouseY, x, y, minimumAdjust, maximumAdjust); };
            var throttledMouseResize = throttle(mouseResize, RESIZE_THROTTLE);
            var withPreventDefault = function (e) {
                moved = true;
                lastX = e.pageX;
                lastY = e.pageY;
                e.preventDefault();
                e.stopImmediatePropagation();
                throttledMouseResize(lastX, lastY);
            };
            var removeListener = function () {
                if (moved) {
                    mouseResize(lastX, lastY);
                }
                window.removeEventListener('mousemove', withPreventDefault);
                window.removeEventListener('mouseup', removeListener);
                onResizeEnd();
            };
            window.addEventListener('mousemove', withPreventDefault);
            window.addEventListener('mouseup', removeListener);
            e.preventDefault();
            e.stopPropagation();
            props.onResizeStart && props.onResizeStart();
        };
        resizeHandle =
            React.createElement(ResizeHandle, { type: resizeType, adjustedSize: space.adjustedSize, width: resizeHandleWidth, height: resizeHandleHeight, onMouseDown: function (e) { return startResize_1(e); }, onTouchStart: function (e) { return startTouchResize_1(e); } });
    }
    var outerStyle = {
        left: (space.left !== undefined ? cssValue(space.left, space.adjustedLeft) : undefined),
        top: (space.top !== undefined ? cssValue(space.top, space.adjustedTop) : undefined),
        right: (space.right !== undefined ? cssValue(space.right, space.adjustedLeft) : undefined),
        bottom: (space.bottom !== undefined ? cssValue(space.bottom, space.adjustedTop) : undefined),
        width: isHorizontalSpace(props.anchor) ? cssValue(props.anchorSize, space.adjustedSize) : space.width,
        height: isVerticalSpace(props.anchor) ? cssValue(props.anchorSize, space.adjustedSize) : space.height,
        zIndex: space.zIndex
    };
    var innerStyle = __assign({}, props.style, {
        left: resizeType === ResizeType.Left && !overlayHandle ? handleSize : undefined,
        top: resizeType === ResizeType.Top && !overlayHandle ? handleSize : undefined,
        right: resizeType === ResizeType.Right && !overlayHandle ? handleSize : undefined,
        bottom: resizeType === ResizeType.Bottom && !overlayHandle ? handleSize : undefined
    });
    var userClasses = props.className ?
        props.className.split(' ').map(function (c) { return c.trim(); }) :
        [];
    var outerClasses = [
        "spaces-space",
        props.scrollable ? (resizeHandle ? "scrollable" : "scrollable-a") : undefined
    ].concat((resizeHandle && props.scrollable ? userClasses.map(function (c) { return c + "-container"; }) : userClasses)).filter(function (c) { return c; });
    var innerClasses = [
        "spaces-space-inner"
    ].concat(userClasses).filter(function (c) { return c; });
    var children = props.children;
    if (props.centerContent === exports.CenterType.Vertical) {
        children = React.createElement(CenteredVertically, null, children);
    }
    else if (props.centerContent === exports.CenterType.HorizontalVertical) {
        children = React.createElement(Centered, null, children);
    }
    return (resizeHandle && props.scrollable ?
        React.createElement(props.as || 'div', {
            id: space.id,
            ref: divElementRef,
            className: outerClasses.join(' '),
            style:  undefined,
            onClick: props.onClick,
            onMouseDown: props.onMouseDown,
            onMouseEnter: props.onMouseEnter,
            onMouseLeave: props.onMouseLeave,
            onMouseMove: props.onMouseMove,
            onTouchStart: props.onTouchStart,
            onTouchMove: props.onTouchMove,
            onTouchEnd: props.onTouchEnd
        }, React.createElement(React.Fragment, null,
             React.createElement(HeadStyles, { spaces: currentContext.children }),
            resizeHandle,
            React.createElement("div", { className: innerClasses.join(' '), style: innerStyle },
                React.createElement(SpaceContext.Provider, { value: currentContext },
                    React.createElement(SpaceInfoContext.Provider, { value: { width: Math.floor(currentSize.width), height: Math.floor(currentSize.height) } }, children))))) :
        React.createElement(props.as || 'div', {
            id: space.id,
            ref: divElementRef,
            className: outerClasses.join(' '),
            style:  innerStyle,
            onClick: props.onClick,
            onMouseDown: props.onMouseDown,
            onMouseEnter: props.onMouseEnter,
            onMouseLeave: props.onMouseLeave,
            onMouseMove: props.onMouseMove,
            onTouchStart: props.onTouchStart,
            onTouchMove: props.onTouchMove,
            onTouchEnd: props.onTouchEnd
        }, React.createElement(React.Fragment, null,
             React.createElement(HeadStyles, { spaces: currentContext.children }),
            resizeHandle,
            React.createElement(SpaceContext.Provider, { value: currentContext },
                React.createElement(SpaceInfoContext.Provider, { value: { width: Math.floor(currentSize.width), height: Math.floor(currentSize.height) } }, children)))));
});
SpaceInternal.propTypes = allProps;

var useParentSpace = function () {
    return {
        startDrag: function () { return null; }
    };
};

exports.Bottom = Bottom;
exports.BottomResizable = BottomResizable;
exports.Centered = Centered;
exports.CenteredVertically = CenteredVertically;
exports.ClearFix = ClearFix;
exports.Fill = Fill;
exports.Fixed = Fixed;
exports.Info = Info;
exports.Layer = Layer;
exports.Left = Left;
exports.LeftResizable = LeftResizable;
exports.Positioned = Positioned;
exports.Right = Right;
exports.RightResizable = RightResizable;
exports.Top = Top;
exports.TopResizable = TopResizable;
exports.ViewPort = ViewPort;
exports.useParentSpace = useParentSpace;
//# sourceMappingURL=index.js.map
